import React from "react"; /**
 * Capture click event outside of the given HTMLElement.
 *
 * @param ref   Reference to the element
 * @param fn    Callback called on ref outside click
 * @param opts  Options
 */
import { Subject, BehaviorSubject } from "rxjs"; /**
 * Wrapper around `rxjs.Subject`.
 *
 * @param subject The subject
 * @param fn The callback used for subscription
 * @returns void
 */
/**
 * Capture click event outside of the given HTMLElement.
 *
 * @param ref   Reference to the element
 * @param fn    Callback called on ref outside click
 * @param opts  Options
 */
type UseClickOutside = (ref: React.RefObject<Node>, fn: ClickOutsideFn, opts?: Partial<ClickOutsideOpts>) => void;
type ClickOutsideFn = (evt: Event) => void;
type ClickOutsideOpts = {
    /**
     * Highest parent where the listener will be attached to.
     *
     * @default document
     */
    root: React.RefObject<Node>;
    /**
     * Blacklisted elements that will not trigger any event.
     *
     * @default []
     */
    except: React.RefObject<Node>[];
    /**
     * Listener event type.
     *
     * @default "click"
     */
    listenerType: keyof DocumentEventMap;
    /**
     * Listener options.
     *
     * @default {passive: true}
     */
    listenerOpts: boolean | AddEventListenerOptions;
};
declare const useClickOutside: UseClickOutside;
type Function = (...args: any[]) => void; /**
 * Add debounce to a handler.
 *
 * @param   fn    Debounced function
 * @param   opts  Debounce options
 * @returns       The handler wrapped
 */
/**
 * Add debounce to a handler.
 *
 * @param   fn    Debounced function
 * @param   opts  Debounce options
 * @returns       The handler wrapped
 */
type UseDebounce = <T extends Function>(fn: T, opts?: number | Partial<DebounceOpts>) => Debounce<T>;
type DebounceOpts = {
    /**
     * Debounce delay before triggering the handler.
     *
     * @default 300
     */
    delay: number;
    /**
     * Should keep the original event by calling `.persist()`.
     *
     * @default false
     */
    persist: boolean;
};
type Debounce<T extends Function> = {
    (...params: Parameters<T>): void;
    /**
     * Abort the debounce.
     */
    abort: () => void;
    /**
     * Abort the debounce and trigger immediatly the handler.
     */
    terminate: () => void;
};
declare const useDebounce: UseDebounce;
type Function$0 = (...args: any[]) => void; /**
 * Wrapper around `setTimeout`.
 *
 * @param   fn    Timed out function
 * @param   opts  Timeout options
 * @returns       Timed out function wrapper
 */
/**
 * Wrapper around `setTimeout`.
 *
 * @param   fn    Timed out function
 * @param   opts  Timeout options
 * @returns       Timed out function wrapper
 */
/**
 * Wrapper around `setTimeout`.
 *
 * @param   fn    Timed out function
 * @param   opts  Timeout options
 * @returns       Timed out function wrapper
 */
type UseTimeout = <T extends Function$0>(fn: T, opts?: number | Partial<TimeoutOpts>) => Timeout<T>;
type TimeoutOpts = {
    /**
     * Timeout delay before triggering the callback.
     *
     * @default 300
     */
    delay: number;
    /**
     * Should keep the original event by calling `.persist()`.
     *
     * @default false
     */
    persist: boolean;
};
type Timeout<T extends Function$0> = {
    (...params: Parameters<T>): void;
    /**
     * Abort the debounce.
     */
    abort: () => void;
    /**
     * Abort the debounce and trigger immediatly the callback.
     */
    terminate: () => void;
};
declare const useTimeout: UseTimeout;
/**
 * A `useState` for booleans.
 *
 * @param defaultVal The default value
 * @returns ToggleState
 */
type UseToggle = (defaultVal?: any) => ToggleState; /**
 * Alias for `useState<boolean>`
 */
/**
 * Alias for `useState<boolean>`
 */
type ToggleState = [boolean, (val?: any) => void];
declare const useToggle: UseToggle;
/**
 * A wrapper around `useInterval`.
 *
 * @param   fn    Interval handler
 * @param   opts  Interval options
 * @returns       A toggle state
 */
type UseInterval = (fn: IntervalFn, opts?: number | Partial<IntervalOpts>) => ToggleState;
type IntervalFn = () => void;
type IntervalOpts = {
    /**
     * `setInterval` delay.
     *
     * @default 1000
     */
    delay: number;
    /**
     * Should start the interval on mount.
     *
     * @default false
     */
    autoStart: boolean;
};
declare const useInterval: UseInterval;
/**
 * A persistant useState, based on React"s `useState` and localForage.
 * Supported drivers: localStorage, WebSQL and IndexedDB.
 */
type UseStoredState = <T>(name: string, opts?: T | Partial<StoredStateOpts<T>>) => [T, (val: T) => Promise<void>, boolean];
type StoredStateOpts<T> = {
    /**
     * Default value.
     */
    defaultVal: T;
    /**
     * LocalForage driver.
     *
     * @default "LOCALSTORAGE"
     */
    driver: StoredStateDriver;
};
type StoredStateDriver = "LOCALSTORAGE" | "WEBSQL" | "INDEXEDDB";
declare const useStoredState: UseStoredState;
/**
 * Wrapper around `rxjs.Subject`.
 *
 * @param subject The subject
 * @param fn The callback used for subscription
 * @returns void
 */
type UseSubject = <T>(subject$: Subject<T>, fn: SubjectFn<T>) => void; /**
 * Callback used for subscription.
 *
 * @param val T
 * @returns void
 */
/**
 * Callback used for subscription.
 *
 * @param val T
 * @returns void
 */
type SubjectFn<T> = (val: T) => void;
declare const useSubject: UseSubject;
/**
 * Wrapper around `rxjs.BehaviorSubject`.
 *
 * @param   subject$  The behavior subject
 * @param   fn        The callback
 * @returns BehaviorSubjectState
 */
type UseBehaviorSubject = <T>(subject$: BehaviorSubject<T>, fn?: BehaviorSubjectFn<T>) => BehaviorSubjectState<T>;
type BehaviorSubjectState<T> = [T, BehaviorSubjectFn<T>]; /**
 * Callback used for subscription.
 *
 * @param val T
 * @returns void
 */
/**
 * Callback used for subscription.
 *
 * @param val T
 * @returns void
 */
type BehaviorSubjectFn<T> = (val: T) => void;
declare const useBehaviorSubject: UseBehaviorSubject;
export { useClickOutside, UseClickOutside, ClickOutsideFn, ClickOutsideOpts, useDebounce, UseDebounce, DebounceOpts, Debounce, useTimeout, UseTimeout, TimeoutOpts, Timeout, useInterval, UseInterval, IntervalFn, IntervalOpts, useStoredState, UseStoredState, StoredStateDriver, StoredStateOpts, useToggle, UseToggle, ToggleState, useSubject, UseSubject, SubjectFn, useBehaviorSubject, UseBehaviorSubject, BehaviorSubjectState, BehaviorSubjectFn };
