'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var invokeMap = _interopDefault(require('lodash/fp/invokeMap'));
var isNumber = _interopDefault(require('lodash/fp/isNumber'));
var noop = _interopDefault(require('lodash/fp/noop'));
var uuid = require('uuid');
var isBoolean = _interopDefault(require('lodash/fp/isBoolean'));
var localForage = _interopDefault(require('localforage'));
var getOr = _interopDefault(require('lodash/fp/getOr'));
var isNil = _interopDefault(require('lodash/fp/isNil'));

const defaultOpts = {
    root: { current: null },
    except: [],
    listenerType: "click",
    listenerOpts: { passive: true },
};

const isOutside = (target, source) => {
    if (!(target instanceof Node))
        return false;
    if (!(source instanceof Node))
        return false;
    if (source.isEqualNode(target))
        return false;
    if (source.contains(target))
        return false;
    return true;
};
const useClickOutside = (ref, fn, overrideOpts = {}) => {
    const opts = Object.assign(defaultOpts, overrideOpts);
    react.useEffect(() => {
        const currRoot = opts.root.current || document;
        const listener = evt => {
            const isBetweenRootAndTarget = [ref, ...opts.except].every(el => {
                const isInsideRoot = !isOutside(evt.target, currRoot);
                const isOutsideOfTarget = isOutside(evt.target, el.current);
                return isInsideRoot && isOutsideOfTarget;
            });
            if (isBetweenRootAndTarget) {
                fn(evt);
            }
        };
        currRoot.addEventListener(opts.listenerType, listener, opts.listenerOpts);
        return () => currRoot.removeEventListener(opts.listenerType, listener, opts.listenerOpts);
    }, [fn, opts, ref]);
};

const defaultOpts$1 = {
    delay: 300,
    persist: false,
};

const useDebounce = (fn, overrideOpts) => {
    const callback = react.useCallback(fn, []);
    const opts = Object.assign(defaultOpts$1, isNumber(overrideOpts) ? { delay: overrideOpts } : overrideOpts);
    const [ready, setReady] = react.useState(false);
    const timeout = react.useRef(0);
    const wrapper = react.useRef(noop);
    const abort = react.useRef(noop);
    const terminate = react.useRef(noop);
    react.useEffect(() => {
        wrapper.current = (...params) => {
            if (opts.persist) {
                invokeMap("persist", params);
            }
            const wrapper = () => {
                callback(...params);
                timeout.current = 0;
            };
            abort.current = () => {
                clearTimeout(timeout.current);
                abort.current = noop;
                terminate.current = noop;
            };
            terminate.current = () => {
                if (timeout.current) {
                    clearTimeout(timeout.current);
                    terminate.current = noop;
                    abort.current = noop;
                    wrapper();
                }
            };
            timeout.current && clearTimeout(timeout.current);
            timeout.current = window.setTimeout(wrapper, opts.delay);
        };
        setReady(true);
    }, [ready, callback, opts.delay, opts.persist]);
    return Object.assign(wrapper.current, {
        abort: () => abort.current(),
        terminate: () => terminate.current(),
    });
};

const defaultOpts$2 = {
    delay: 300,
    persist: false,
};

const useTimeout = (fn, overrideOpts) => {
    const callback = react.useCallback(fn, []);
    const opts = Object.assign(defaultOpts$2, isNumber(overrideOpts) ? { delay: overrideOpts } : overrideOpts);
    const [ready, setReady] = react.useState(false);
    const timeouts = react.useRef(new Map());
    const wrapper = react.useRef(noop);
    const abort = react.useRef(noop);
    const terminate = react.useRef(noop);
    react.useEffect(() => {
        wrapper.current = (...params) => {
            if (opts.persist) {
                invokeMap("persist", params);
            }
            const id = uuid.v4();
            const wrapper = () => {
                callback(...params);
                timeouts.current.delete(id);
            };
            abort.current = () => {
                timeouts.current.forEach(([t]) => t && clearTimeout(t));
                timeouts.current.clear();
                abort.current = noop;
                terminate.current = noop;
            };
            terminate.current = () => {
                timeouts.current.forEach(([t, wrapper]) => {
                    if (t) {
                        clearTimeout(t);
                        terminate.current = noop;
                        abort.current = noop;
                        wrapper();
                    }
                });
                timeouts.current.clear();
                abort.current = noop;
                terminate.current = noop;
            };
            timeouts.current.set(id, [window.setTimeout(wrapper, opts.delay), wrapper]);
        };
        setReady(true);
    }, [ready, callback, opts.delay, opts.persist]);
    return Object.assign(wrapper.current, {
        abort: () => abort.current(),
        terminate: () => terminate.current(),
    });
};

const useToggle = defaultVal => {
    const [val, setVal] = react.useState(defaultVal === true);
    function toggler(override) {
        setVal(isBoolean(override) ? override : !val);
    }
    return [val, toggler];
};

const defaultOpts$3 = {
    delay: 1000,
    autoStart: false,
};

const useInterval = (fn, overrideOpts) => {
    const callback = react.useCallback(fn, []);
    const opts = Object.assign(defaultOpts$3, isNumber(overrideOpts) ? { delay: overrideOpts } : overrideOpts);
    const timeout = react.useRef(0);
    const [isIntervalOn, toggleInterval] = useToggle(opts.autoStart);
    react.useEffect(() => {
        function clearTimeoutSafe() {
            clearTimeout(timeout.current);
            timeout.current = 0;
        }
        if (isIntervalOn) {
            timeout.current = window.setInterval(callback, opts.delay);
        }
        else {
            clearTimeoutSafe();
        }
        return () => {
            clearTimeoutSafe();
        };
    }, [callback, isIntervalOn, opts.delay]);
    return [isIntervalOn, toggleInterval];
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            }
        }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const defaultDriver = "LOCALSTORAGE";

const useStoredState = (name, overrideOpts) => {
    const defaultVal = getOr(overrideOpts, "defaultVal", overrideOpts);
    const opts = Object.assign({ defaultVal, driver: defaultDriver }, overrideOpts);
    const storage = react.useRef(null);
    const [isReady, setReady] = react.useState(false);
    const [val, setVal] = react.useState(defaultVal);
    const updateVal = react.useCallback((val) => __awaiter(void 0, void 0, void 0, function* () {
        if (storage.current) {
            setVal(val);
            yield storage.current.setItem(name, val);
        }
    }), [name]);
    react.useEffect(() => {
        if (!isReady) {
            storage.current = localForage.createInstance({ name, driver: localForage[opts.driver] });
            storage.current
                .getItem(name)
                .then(storedVal => (isNil(storedVal) ? updateVal(defaultVal) : setVal(storedVal)))
                .then(() => setReady(true));
        }
    }, [defaultVal, isReady, name, opts.driver, updateVal]);
    return [val, updateVal, isReady];
};

const useSubject = (subject, fn) => {
    const callback = react.useCallback(fn, []);
    react.useEffect(() => {
        const subscription = subject.subscribe(callback);
        return () => subscription.unsubscribe();
    }, [callback, subject]);
};

const useBehaviorSubject = (subject$, fn) => {
    const callback = react.useCallback(fn || noop, []);
    const [val, setVal] = react.useState(subject$.value);
    react.useEffect(() => {
        const subscription = subject$.subscribe(val => {
            callback(val);
            setVal(val);
        });
        return () => {
            subscription.unsubscribe();
        };
    }, [callback, subject$]);
    return [val, subject$.next.bind(subject$)];
};

exports.useBehaviorSubject = useBehaviorSubject;
exports.useClickOutside = useClickOutside;
exports.useDebounce = useDebounce;
exports.useInterval = useInterval;
exports.useStoredState = useStoredState;
exports.useSubject = useSubject;
exports.useTimeout = useTimeout;
exports.useToggle = useToggle;
//# sourceMappingURL=index.cjs.js.map
